#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-08-07
# Last Modified: $Date: 2008-11-28 11:32:25 +0000 (Fri, 28 Nov 2008) $
# Id:            $Id: merge.pm 3625 2008-11-28 11:32:25Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-4.0/lib/npg_common/image/merge.pm $
#
package npg_common::image::merge;
use strict;
use warnings;
use English qw{-no_match_vars};
use GD;
use GD::Image;
use Sanger::Graphics::ColourMap;
use base qw(npg_common::image);
use npg_common::image::graph;
use Readonly;
use Carp;
use POSIX qw(floor ceil);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3625 $ =~ /(\d+)/mx; $r; };

Readonly our $GRAPH_HEIGHT_DEFAULT      => 200;
Readonly our $GRAPH_WIDTH_DEFAULT       => 400;
Readonly our $HEIGHT_OF_COLUMN_HEADINGS_TP => 50;
Readonly our $WIDTH_OF_ROW_HEADINGS_TP     => 60;
Readonly our $RIGHT_MARGIN_TP              => 25;
Readonly our $BOTTOM_MARGIN_TP             => 40;
Readonly our $HEIGHT_OF_COLUMN_HEADINGS_TL => 40;
Readonly our $WIDTH_OF_ROW_HEADINGS_TL     => 80;
Readonly our $RIGHT_MARGIN_TL              => 65;
Readonly our $BOTTOM_MARGIN_TL             => 25;
Readonly our $THIRD                     => 3;
Readonly our $QUARTER                   => 4;
Readonly our $EXTRA_FIVE_PIXELS         => 5;
Readonly our $EXTRA_EIGHT_PIXELS        => 8;

sub allowed_formats {
  my ($self, $format) = @_;
  my $allowed_formats = {
    table_portrait => 1,
    table_landscape => 1,
  };
  if (!$allowed_formats->{$format}) {
    croak "$format is not a supported format of merged images";
  }
  return 1;
}

sub merge_images {
  my ($self, $arg_refs) = @_;
  my $format = $arg_refs->{format};
  $self->allowed_formats($format);
  return $self->$format($arg_refs);
}

sub table_portrait {
  my ($self, $arg_refs) = @_;

  my $data           = $arg_refs->{data};

  if (!$data || !$data->[0]) {
    croak 'no data provided for drawing images';
  }

  my $col_headings   = $arg_refs->{col_headings};
  my $row_headings   = $arg_refs->{row_headings};
  my $args_for_image = $arg_refs->{args_for_image};
  my $titles         = $arg_refs->{titles};
  my $y_min_values   = $arg_refs->{y_min_values};
  my $y_max_values   = $arg_refs->{y_max_values};
  my $y_labels       = $arg_refs->{y_labels};

  my $rows = scalar@{$data} || 0;
  my $cols = scalar@{$data->[0]} || 0;

  $args_for_image->{return_object} = 1;

  my $row_count = 0;
  foreach my $row (@{$data}) {
    my $col_count = 0;
    foreach my $col (@{$row}) {
      my $args = \%{$args_for_image};

      if ($titles) {
        $args->{title} = $titles->[$col_count];
      }
      if ($y_min_values) {
        $args->{y_min_value} = $y_min_values->[$col_count];
      }
      if ($y_max_values) {
        $args->{y_max_value} = $y_max_values->[$col_count];
      }
      if ($y_labels) {
        $args->{y_label} = $y_labels->[$col_count];
      }

      eval {
        my $image = npg_common::image::graph->new();
        $col = $image->plotter($col, $args, q{lines}, 1);
      } or do {
        croak $EVAL_ERROR;
      };

      $col_count++;
    }
    $row_count++;
  }

  my $height_of_image = $args_for_image->{height} || $GRAPH_HEIGHT_DEFAULT;
  my $width_of_image  = $args_for_image->{width}  || $GRAPH_WIDTH_DEFAULT;

  my $height = $rows * $height_of_image + $HEIGHT_OF_COLUMN_HEADINGS_TP + $BOTTOM_MARGIN_TP;
  my $width  = $cols * $width_of_image  + $WIDTH_OF_ROW_HEADINGS_TP + $RIGHT_MARGIN_TP;

  my $im = new GD::Image($width,$height);

  $row_count = 0;
  foreach my $row (@{$data}) {
    my $col_count = 0;
    foreach my $col (@{$row}) {
      my $x_pos = $WIDTH_OF_ROW_HEADINGS_TP + 2 + $col_count*($col->width() + $EXTRA_EIGHT_PIXELS);
      my $y_pos = $HEIGHT_OF_COLUMN_HEADINGS_TP + $row_count*($col->height() + $EXTRA_FIVE_PIXELS);

# uncomment if you want positions for an image map to be outputed
#      my $x_final = $x_pos + $col->width();
#      my $y_final = $y_pos + $col->height();
#      print qq{<area coords="($x_pos $y_pos $x_final $y_final)[0..3]" href="" title="">\n};

      $im->copy($col, $x_pos, $y_pos, 0,0,$width_of_image,$height_of_image);
      $col_count++;
      $arg_refs->{height_of_image} = $col->height();
      $arg_refs->{width_of_image}  = $col->width();
    }
    $row_count++;
  }
  $self->draw_table_text_tp($arg_refs, $im);
  $self->draw_table_borders_tp($arg_refs, $im);

#open (FH, ">:raw", 'image.png') || croak 'could not open';print FH $im->png;close FH;
  return $im->png();

}

sub draw_table_text_tp {
  my ($self, $arg_refs, $im) = @_;

  my $width  = $im->width();
  my $height = $im->height();

  my $text_colour = $im->colorAllocate(0,0,0);

  my $column_headings = $arg_refs->{column_headings};
  my $row_headings = $arg_refs->{row_headings};

  my $height_of_image = $arg_refs->{height_of_image};
  my $width_of_image  = $arg_refs->{width_of_image};

  my $count = 0;
  foreach my $text (@{$column_headings}) {
    my $x = $WIDTH_OF_ROW_HEADINGS_TP/$QUARTER + ($width_of_image*$count);
    if ($count > 0) { $x -= $width_of_image/2; };
    my $y = $HEIGHT_OF_COLUMN_HEADINGS_TP/2;
    $im->string(gdGiantFont, $x, $y, $text, $text_colour);
    $count++;
  }
  $count = 1;
  foreach my $text (@{$row_headings}) {
    my $x = $WIDTH_OF_ROW_HEADINGS_TP/2;
    my $y = $HEIGHT_OF_COLUMN_HEADINGS_TP + $height_of_image*$count - $height_of_image/2;
    $im->string(gdGiantFont, $x, $y, $text, $text_colour);
    $count++;
  }

  return 1;
}

sub draw_table_borders_tp {
  my ($self, $arg_refs, $im) = @_;

  my $width  = $im->width();
  my $height = $im->height();

  my $border_colour = $im->colorAllocate(0,0,0);

  my $height_of_image = $arg_refs->{height_of_image};
  my $width_of_image  = $arg_refs->{width_of_image};
  my $column_headings = $arg_refs->{column_headings};
  my $row_headings = $arg_refs->{row_headings};

  $im->rectangle(0,0,$width-1,$height-1,$border_colour);

  my $count = 0;
  foreach my $heading (@{$column_headings}) {
    $im->rectangle(($WIDTH_OF_ROW_HEADINGS_TP + $count * ($width_of_image + $EXTRA_EIGHT_PIXELS) - 1), 0, ($WIDTH_OF_ROW_HEADINGS_TP + $count * ($width_of_image + $EXTRA_EIGHT_PIXELS)), $height, $border_colour);
    $count++;
  }
  $count = 0;
  foreach my $heading (@{$row_headings}) {
    $im->rectangle(0,($HEIGHT_OF_COLUMN_HEADINGS_TP + $count * ($height_of_image + $EXTRA_FIVE_PIXELS) - 2), $width, ($HEIGHT_OF_COLUMN_HEADINGS_TP + $count * ($height_of_image + $EXTRA_FIVE_PIXELS) - 1), $border_colour);
    $count++;
  }
  return 1;
}
sub draw_table_borders_tl {
  my ($self, $arg_refs, $im) = @_;

  my $width  = $im->width();
  my $height = $im->height();

  my $border_colour = $im->colorAllocate(0,0,0);

  my $height_of_image = $arg_refs->{height_of_image};
  my $width_of_image  = $arg_refs->{width_of_image};
  my $column_headings = $arg_refs->{column_headings};
  my $row_headings    = $arg_refs->{row_headings};

  $im->rectangle(0,0,$width-1,$height-1,$border_colour);

  my $count = 0;
  foreach my $heading (@{$column_headings}) {
    $im->rectangle(($WIDTH_OF_ROW_HEADINGS_TL + $count * ($width_of_image + $EXTRA_EIGHT_PIXELS) - 1), 0, ($WIDTH_OF_ROW_HEADINGS_TL + $count * ($width_of_image + $EXTRA_EIGHT_PIXELS)), $height, $border_colour);
    $count++;
  }
  $count = 0;
  foreach my $heading (@{$row_headings}) {
    $im->rectangle(0,($HEIGHT_OF_COLUMN_HEADINGS_TL + $count * ($height_of_image + $EXTRA_FIVE_PIXELS) - 2), $width, ($HEIGHT_OF_COLUMN_HEADINGS_TL + $count * ($height_of_image + $EXTRA_FIVE_PIXELS) - 1), $border_colour);
    $count++;
  }
  return 1;
}

sub draw_table_text_tl {
  my ($self, $arg_refs, $im) = @_;

  my $width  = $im->width();
  my $height = $im->height();

  my $text_colour = $im->colorAllocate(0,0,0);

  my $column_headings = $arg_refs->{column_headings};
  my $row_headings    = $arg_refs->{row_headings};

  my $height_of_image = $arg_refs->{height_of_image};
  my $width_of_image  = $arg_refs->{width_of_image};

  my $count = 1;
  my @column_headings = @{$column_headings};
  my $row_heading_space_heading = shift @column_headings;
  my $x = $WIDTH_OF_ROW_HEADINGS_TL/$QUARTER;
  my $y = $HEIGHT_OF_COLUMN_HEADINGS_TL/2;
  $im->string(gdGiantFont, $x, $y, $row_heading_space_heading, $text_colour);
  foreach my $text (@column_headings) {
    my $x = $WIDTH_OF_ROW_HEADINGS_TL + ($width_of_image*$count) - ($width_of_image/2) + (($count - 1) * $EXTRA_EIGHT_PIXELS);
    $im->string(gdGiantFont, $x, $y, $text, $text_colour);
    $count++;
  }
  $count = 1;
  foreach my $text (@{$row_headings}) {
    my @temp = split /\s+/xms, $text;
    my $y = $HEIGHT_OF_COLUMN_HEADINGS_TL + $height_of_image*$count - $height_of_image/2;
    if (scalar@temp == 1) {
      $im->string(gdGiantFont, $x, $y, $text, $text_colour);
    } else {
      my $text_1 = $temp[0]. q{ }. $temp[1];
      $im->string(gdGiantFont, $x, $y, $text_1, $text_colour);
      my $text_2 = $temp[2];
      $y = $y + $height_of_image/$THIRD;
      $im->string(gdGiantFont, $x, $y, $text_2, $text_colour);
    }
    $count++;
  }

  return 1;
}

sub table_landscape {
  my ($self, $arg_refs) = @_;

  my $data           = $arg_refs->{data};

  if (!$data || !$data->[0]) {
    croak 'no data provided for drawing images';
  }

  my $col_headings   = $arg_refs->{col_headings};
  my $row_headings   = $arg_refs->{row_headings};
  my $args_for_image = $arg_refs->{args_for_image};
  my $titles         = $arg_refs->{titles};
  my $y_min_values   = $arg_refs->{y_min_values};
  my $y_max_values   = $arg_refs->{y_max_values};
  my $y_labels       = $arg_refs->{y_labels};

  my $rows = scalar@{$data} || 0;
  my $cols = scalar@{$data->[0]} || 0;

  $args_for_image->{return_object} = 1;

  my $row_count = 0;
  foreach my $row (@{$data}) {
    my $col_count = 0;
    foreach my $col (@{$row}) {
      my $args = \%{$args_for_image};

      if ($titles) {
        $args->{title} = $titles->[$row_count];
      }
      if ($y_min_values) {
        $args->{y_min_value} = $y_min_values->[$row_count];
      }
      if ($y_max_values) {
        $args->{y_max_value} = $y_max_values->[$row_count];
      }
      if ($y_labels) {
        $args->{y_label} = $y_labels->[$row_count];
      }

      eval {
        my $image = npg_common::image::graph->new();
        $col = $image->plotter($col, $args, q{lines}, 1);
      } or do {
        croak $EVAL_ERROR;
      };

      $col_count++;
    }
    $row_count++;
  }

  my $height_of_image = $args_for_image->{height} || $GRAPH_HEIGHT_DEFAULT;
  my $width_of_image  = $args_for_image->{width}  || $GRAPH_WIDTH_DEFAULT;

  my $height = $rows * $height_of_image + $HEIGHT_OF_COLUMN_HEADINGS_TL + $BOTTOM_MARGIN_TL;
  my $width  = $cols * $width_of_image  + $WIDTH_OF_ROW_HEADINGS_TL + $RIGHT_MARGIN_TL;

  my $im = new GD::Image($width,$height);

  $row_count = 0;
  foreach my $row (@{$data}) {
    my $col_count = 0;
    foreach my $col (@{$row}) {
      my $x_pos = $WIDTH_OF_ROW_HEADINGS_TL + 2 + $col_count*($col->width() + $EXTRA_EIGHT_PIXELS);
      my $y_pos = $HEIGHT_OF_COLUMN_HEADINGS_TL + $row_count*($col->height() + $EXTRA_FIVE_PIXELS);

# uncomment if you want positions for an image map to be outputed
#      my $x_final = $x_pos + $col->width();
#      my $y_final = $y_pos + $col->height();
#      print qq{<area coords="($x_pos $y_pos $x_final $y_final)[0..3]" href="" title="">\n};
      
      $im->copy($col, $x_pos, $y_pos, 0,0,$width_of_image,$height_of_image);
      $col_count++;
      $arg_refs->{height_of_image} = $col->height();
      $arg_refs->{width_of_image}  = $col->width();
    }
    $row_count++;
  }
  $self->draw_table_text_tl($arg_refs, $im);
  $self->draw_table_borders_tl($arg_refs, $im);

#open (FH, ">:raw", 'image.png') || croak 'could not open';print FH $im->png;close FH;
  return $im->png();

}
1;

__END__

=head1 NAME

npg_common::image::merge

=head1 VERSION

$LastChangedRevision: 3625 $

=head1 SYNOPSIS

  my $oImageMerge = npg_common::image::merge->new({});

=head1 DESCRIPTION

Wrapper object to provide a scale image showing the colours in a linear gradient and the min and max values;

=head1 SUBROUTINES/METHODS

=head2 new - constructor to create a graph object

=head2 build_linear_gradient - routine to obtain a scale of colours from given colours. Defaults from black through yellow to red

=head2 merge_images - default method to call

=head2 allowed_formats - checks supplied format to merge into is an allowed format

=head2 draw_table_borders_tp - draws borders on the image so a table style view appears
=head2 draw_table_text_tp - Puts headers into the image to look like a table
=head2 table_portrait - method table creates a table-style merged image
=head2 draw_table_borders_tl - draws borders on the image so a table style view appears
=head2 draw_table_text_tl - Puts headers into the image to look like a table
=head2 table_landscape - method table creates a table-style merged image

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

GD
GD::Image
Sanger::Graphics::ColourMap
Class::Accessor
npg_common::image
npg_common::image::graph

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
