#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-09-01
# Last Modified: $Date: 2008-11-28 11:32:25 +0000 (Fri, 28 Nov 2008) $
# Id:            $Id: image_map.pm 3625 2008-11-28 11:32:25Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-4.0/lib/npg_common/image_map.pm $
#
package npg_common::image_map;
use strict;
use warnings;
use base qw(Class::Accessor);
use Readonly;
use Carp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3625 $ =~ /(\d+)/mx; $r; };

Readonly our $MAX_COORDINATES  => 3;
__PACKAGE__->mk_accessors(qw(image_refs));


sub new {
  my ($class, $ref) = @_;
  $ref ||= {};
  bless $ref, $class;

  return $ref;
}

sub render_map {
  my ($self, $arg_refs) = @_;
  
  my $data = $arg_refs->{data};
  my $image_url  = $arg_refs->{image_url};
  my $map = qq[<map name="$arg_refs->{id}">\n];
  for my $box (@{$data}) {
    my $data_information = pop@{$box};
    my $url = $data_information->{url} || q{#};
    my $title;
    foreach my $key (sort keys %{$data_information}) {
      next if $key eq 'url';
      my $name = ucfirst$key;
      $title .= "$name:$data_information->{$key} "
    }
    $map .= qq[<area coords="(@{$box})[0..$MAX_COORDINATES]" href="$url" title="$title"/>\n];
  }
  $map .= qq[</map><img src="$image_url" usemap="#$arg_refs->{id}"/>];
  
  return $map;
}

1;

__END__

=head1 NAME

npg_common::image_map

=head1 VERSION

$LastChangedRevision: 3625 $

=head1 SYNOPSIS

  my $oImageMap = npg_common::image_map->new();

=head1 DESCRIPTION

Class to provide an image map reference to overlay an image (likely heatmaps)

=head1 SUBROUTINES/METHODS

=head2 new - constructor to create an image_map object

=head2 render_map - renders the data points and id into a html map for use in front of the image. For the data sets, you must have a hash, the key value pairs for a title for the image space

  my $sRenderMap = $oImageMap->render_map($arg_refs);

  $arg_refs = {
    data => [[$x1,$y1,$x2,$y2,{url => $a_url, other_key_value_pairs}],..],
    image_url => $url_of_image_map_is_for,
    id => $unique_id_for_map,
  };

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Class::Accessor
Readonly
Carp

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
