#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-08-07
# Last Modified: $Date: 2008-11-28 11:32:25 +0000 (Fri, 28 Nov 2008) $
# Id:            $Id: 10-image_merge.t 3625 2008-11-28 11:32:25Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-4.0/t/10-image_merge.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use SangerPaths qw{core ensembl badger trackingnpg tracking};
use Test::More 'no_plan'; #tests => 17;
use Digest::MD5;

my $md5_object = Digest::MD5->new();

use_ok('npg_common::image::merge');

{
  my $merge = npg_common::image::merge->new({});
  isa_ok($merge, 'npg_common::image::merge', '$merge');
  my $arg_refs = {
     format => 'spamcodjfijdijcvdocv',
 };
  eval { $merge->merge_images($arg_refs); };
  like($EVAL_ERROR, qr{spamcodjfijdijcvdocv\ is\ not\ a\ supported\ format\ of\ merged\ images}, 'croaked on running $merge->merge_images() as unsupported format');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
     format => 'table_portrait',
 };
  eval { $merge->merge_images($arg_refs); };
  like($EVAL_ERROR, qr{no\ data\ provided\ for\ drawing\ images}, 'croaked on running $merge->merge_images() as no data provided');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
     format => 'table_portrait',
     data   => [],
 };
  eval { $merge->merge_images($arg_refs); };
  like($EVAL_ERROR, qr{no\ data\ provided\ for\ drawing\ images}, 'croaked on running $merge->merge_images() as no column data provided');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
    format => 'table_portrait',
    rows   => '8',
    cols   => '3',
    data   => [
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ]
    ],
    column_headings => ['Lane', 'All', 'Call', '% Base Calls'],
    y_min_values => [0,0,0],
    y_max_values => [1500,1500,undef],
    
    row_headings => [qw(1 2 3 4 5 6 7 8)],
    args_for_image => {
      'width'        => 126,
      'height'       => 84,
      'x_labels_vertical' => 1,
    },
  };
  my $png;
  eval { $png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{2bf6133eb97ae612c2ca9a2fc8856ca6}, 'md5 hex digest of png is correct');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
    format => 'table_landscape',
    rows   => '3',
    cols   => '8',
    data   => [
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    ],
    row_headings => ['All', 'Call', '% Base Calls'],
    y_min_values => [0,0,0],
    y_max_values => [1500,1500,undef],
    
    column_headings => [qw(Lane 1 2 3 4 5 6 7 8)],
    args_for_image => {
      'width'        => 126,
      'height'       => 84,
      'x_labels_vertical' => 1,
    },
  };
  my $png;
  eval { $png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{7feb32172ae5b7d9d86cc655f80f328d}, 'md5 hex digest of png is correct');
}
__END__  
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ]
